<?php

namespace Drupal\html_to_pdf\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Dompdf\Dompdf;
use Dompdf\Options;
use DOMDocument;
use Symfony\Component\HttpFoundation\Response;


class HtmlUploadForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pdf_upload_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'html_to_pdf.settings',
    ];
  }

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * URL generator for file paths.
   *
   * @var \Drupal\file\Entity\FileUrlGenerator
   */
  protected $fileUrlGenerator;
 
  public function __construct(EntityTypeManagerInterface $entity_type_manager, $file_url_generator ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fileUrlGenerator = $file_url_generator;
    
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
        $container->get('entity_type.manager'),
        $container->get('file_url_generator'),
      
      );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('html_to_pdf.settings');

    $form['file_upload'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Upload HTML File'),
      '#upload_location' => 'public://',
      '#default_value' => $config->get('file_upload'),
      '#upload_validators' => [
        'file_validate_extensions' => ['html'],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('html_to_pdf.settings');
    $file = $form_state->getValue('file_upload');

    if (!empty($file[0])) {
        $file_attachment = $this->entityTypeManager->getStorage('file')->load($file[0]);
        $filename = $file_attachment->getFilename();
        $file_attachment->setPermanent();
        $file_attachment->save();

        $url = $this->fileUrlGenerator->generate($file_attachment->getFileUri());
        $filePath = $url->getOption('path');

        $this->generatePdf($filePath, $filename);
        parent::submitForm($form, $form_state);
    }
}

protected function generatePdf($filePath, $filename) {
    $html_file_content = file_get_contents($filePath);

    if ($this->isValidHTML($html_file_content)) {
        $filenamearr = explode(".", $filename);
        $options = new \Dompdf\Options();
        $options->set('isHtml5ParserEnabled', true);
        // Other options...
        
        $dompdf = new \Dompdf\Dompdf($options);
        $dompdf->loadHtml($html_file_content);
        $dompdf->setPaper('A4', 'portrait');
        $dompdf->render();

        header('Content-Type: application/pdf');
        header('Content-Disposition: attachment; filename="' . $filenamearr[0] . '.pdf"');
        echo $dompdf->output();
        exit();
    } else {
        \Drupal::logger('html_to_pdf')->error('Invalid characters detected in the HTML file.');
        \Drupal::messenger()->addError($this->t('The uploaded file contains invalid characters.'));
    }
}

function isValidHTML($html_content) {
    $dom = new DOMDocument();
    libxml_use_internal_errors(true);
    $dom->loadHTML($html_content, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);

    $errors = libxml_get_errors();
    libxml_clear_errors();

    return empty($errors);
}

}